@@ -9,7 +9,7 @@ use POSIX        qw( EINTR EWOULDBLOCK );
 use IO::Select ();
 
 our @ISA = qw();
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 # TO DO (wish list):
 # Restart crashed child workers.
@@ -20,6 +20,7 @@ sub new {
   my $self = {
     work_handler    => $attrs{work_handler},                # required
     init_handler    => $attrs{init_handler}   || undef,     # optional
+    exit_handler    => $attrs{exit_handler}   || undef,     # optional
     result_handler  => $attrs{result_handler} || undef,     # optional
     worker_count    => $attrs{worker_count}   || 3,         # optional, how many child workers
     global_timeout  => $attrs{global_timeout} || 0,         # optional, in seconds, 0 is unlimited
@@ -165,21 +166,41 @@ sub shut_down {
 # Make socket blocking/nonblocking
 sub blocking {
   my ($self, $socket, $makeblocking)=@_;
-  my $flags = fcntl($socket, F_GETFL, 0)
-    or croak("fcntl failed: $!");
-  my $blocking = ($flags & O_NONBLOCK) == 0;
-  if ($blocking  && ! $makeblocking) {
-    $flags |= O_NONBLOCK;
-  } elsif (! $blocking && $makeblocking) {
-    $flags &= ~O_NONBLOCK;
-  } else {
-    # do nothing
-    return $blocking;
-  }
+
+  ### --- W I N D O W S --- ###
+  if ($^O eq 'MSWin32') {
+    # ioctl() requires a pointer to a long, containing the nonblocking value.
+
+    # The long var
+    my $nonblocking = pack('L', $makeblocking ? 0 : 1);
+
+    # The pointer to it
+    my $plong = unpack('I', pack('P', $nonblocking));
+
+    # The nonblocking request. FIONBIO is 0x8004667e
+    ioctl($socket, 0x8004667e, $plong)
+      or croak("ioctl failed: $!");
   
-  fcntl($socket, F_SETFL, $flags)
-    or croak("fcntl failed: $!");
-  return $blocking;
+
+  ### --- LINUX, BSD, etc --- ###
+  } else {
+    my $flags = fcntl($socket, F_GETFL, 0)
+      or croak("fcntl failed: $!");
+    my $blocking = ($flags & O_NONBLOCK) == 0;
+    if ($blocking  && ! $makeblocking) {
+      $flags |= O_NONBLOCK;
+    } elsif (! $blocking && $makeblocking) {
+      $flags &= ~O_NONBLOCK;
+    } else {
+      # do nothing
+      return;
+    }
+
+    fcntl($socket, F_SETFL, $flags)
+      or croak("fcntl failed: $!");
+  }
+
+  return;
 }
 
 # -----------------------------------------------------------------
@@ -674,8 +695,16 @@ sub worker_loop {
       }
     }
   };
-  if ($@) {
-    croak($@);
+  my $errm = $@ || '';
+  eval {
+  if ($self->{exit_handler}) {
+    $self->log("worker_loop: calling exit_handler()\n");
+    $self->{exit_handler}->();
+  }
+  };
+  $errm .= "\n$@" if $@;
+  if ($errm) {
+    croak($errm);
   }
 }
 
@@ -756,6 +785,8 @@ The main process can collect available responses from the Boss, and/or send it m
 
 In general, it's a good idea to construct the object early in a program's life, before any threads are spawned, and before much memory is allocated, as the Boss, and each Worker will inherit the memory footprint.
 
+The 0.09 release includes Windows compatibility! (see Credits below)
+
 =head1 METHODS
 
 =head2 new(...)
@@ -766,6 +797,7 @@ Creates and returns a new Parallel::Fork::BossWorkerAsync object.
     work_handler    => \&work_sub,
     result_handler  => \&result_sub,
     init_handler    => \&init_sub,
+    exit_handler    => \&exit_sub,
     global_timeout  => 0,
     worker_count    => 3,
     msg_delimiter   => "\0\0\0",
@@ -792,6 +824,10 @@ The result_handler is expected to return a hashref.
 
 The init_handler argument is optional.  The referenced function receives no arguments and returns nothing.  It is called only once by each worker, just after it's forked off from the boss, and before entering the job processing loop. This subroutine is not affected by the value of global_timeout.  This could be used to connect to a database, instantiate a non-shared object, etc.
 
+=item * C<< exit_handler => \&exit_sub >>
+
+The exit_handler argument is optional.  The referenced function receives no arguments and returns nothing.  It is called only once by each worker, just before exiting.  This subroutine is not affected by the value of global_timeout.  This could be used to disconnect from a database, etc.
+
 =item * C<< global_timeout => $seconds >>
 
 By default, a handler can execute forever. If global_timeout is specified, an alarm is setup to terminate the work_handler so processing can continue.
@@ -865,8 +901,6 @@ If global_timeout is set, and a timeout occurs, the worker returns:
 
 Please report bugs to jvann.cpan@gmail.com.
 
-This module will probably not work on Windows due to its reliance on UNIX IPC mechanisms.
-
 The Boss and Worker processes are long-lived. There is no restart mechanism for processes that exit prematurely. If it's the Boss, you're dead anyway, but if it's one or more Workers, the app will continue running, but throughput will suck. 
 
 The code should in some way overcome the tiny socket buffer limitations of BSD operating systems. Unbuffered reads are limited to 8192 byte chunks. If you pass megabytes of data with each job, the processing will not fail, but it will seem to be hung -- it can get VERY slow! This is not an issue on Linux, and will not be a problem on BSD if you pass less then say, 64k, between processes. If you know how to force an unbuffered socket read to use an arbitrarily large buffer (1 megabyte, for example), please shoot me an email.
@@ -879,6 +913,8 @@ Jeff Rodriguez: wrote the module Parallel::Fork::BossWorker, which inspired this
 
 Rob Navarro: reported -- and fixed! -- errors in fork() error handling, and in the reaping of dead child processes.
 
+Mario Roy: contributed the Windows socket code.
+
 =head1 COPYRIGHT AND LICENSE
 
 Copyright (C) 2009-2013 by joe vannucci, E<lt>jvann.cpan@gmail.comE<gt>
